--------------------------------------------------------------
-- Classic snake game
-- Original code
-- 2009 Led Lab @PUC-Rio www.eluaproject.net
-- Dado Sutter
-- Ives Negreiros
-- To Benjamin
--
-- Modifications CMc 20161004  Changes to init function so lcd.clear() works on Taranis
--                         Also some editorial "cleanup"
-- Modifications CMc 20161005  Changes detect Head colliding with TAIL
-- Modifications CMc 20161006  Changes Provide Speed changes for the Snake
-- Modifications CMc 20170203  Changes Provide Graphics instead of ASCII art
-- Calvin McCarthy  (Boomerwing on RC Groups)
---------------------------------------------------------------
local LCD_W = 212
local LCD_H = 64
local xMax = math.floor(( LCD_W / 6 ) - 1)
local yMax = math.floor(( LCD_H / 8 ) - 1)
local game_map = {}
local snakeSpeed = 45
local speedMin = 60
local speedMax = 10
local speedStep = 4
local Head = {}
local Tail = {}

local highscore = 0
local size = 3
Tail.x = 1
Tail.y = 1
Head.x = Tail.x + ( size - 1 )
Head.y = Tail.y

local Food = {}
Food.x = false
Food.y = false

Head.dx = 1
Head.dy = 0
Tail.dx = Head.dx
Tail.dy = Head.dy
local direction = "right"
local score = 0
local timeStart = nil
local TimeNow = nil
local exitLoop = false
    
-- ************************************************************************************************

local function create_food()
  Food.x = math.random( xMax - 1) 
  Food.y = math.random( yMax - 1)
  while game_map[ Food.x ][ Food.y ] do
    Food.x = math.random( xMax - 1 )
    Food.y = math.random( yMax - 1 )
  end

  game_map[ Food.x ][ Food.y ] = "food"
     lcd.drawPixmap(Food.x * 6, Food.y* 8,"/SCRIPTS/TELEMETRY/bug.bmp" )     -- new CMc 20170203
end

-- ************************************************************************************************

local function eat_food()
     lcd.drawPixmap(Food.x * 6, Food.y * 8,"/SCRIPTS/TELEMETRY/snakehd.bmp" )    -- new CMc 20170203
  game_map[ Head.x ][ Head.y ] = nil
  create_food()
  score = score + 1
end

-- ************************************************************************************************

local function check_collision()
  if Head.x < 0 or Head.x > xMax then
    return true
  elseif Head.y < 0 or Head.y > yMax then
    return true
  elseif ( ( game_map[ Head.x ][ Head.y ] ) ~= nil and ( game_map[ Head.x ][ Head.y ] ~= "food" )) then
    return true 
  end
  return false 

end

-- ************************************************************************************************

local function getDirection()
  local dir = direction
  if getValue('rud') > 100 and direction ~= "left" then
    dir = "right"
    Head.dx = 1
    Head.dy = 0
  end
  if getValue('rud') < -100 and direction ~= "right" then
    dir = "left"
    Head.dx = -1
    Head.dy = 0
  end
  if getValue('ele') > 100 and direction ~= "down" then
    dir = "up"
    Head.dx = 0
    Head.dy = -1
  end
  if getValue('ele') < -100 and direction ~= "up" then
    dir = "down"
    Head.dx = 0
    Head.dy = 1
  end
  direction = dir
 end


-- ************************************************************************************************

local function move()
  if game_map[ Tail.x ][ Tail.y ] == "right" then
    Tail.dx = 1
    Tail.dy = 0
  elseif game_map[ Tail.x ][ Tail.y ] == "left" then
    Tail.dx = -1
    Tail.dy = 0
  elseif game_map[ Tail.x ][ Tail.y ] == "up" then
    Tail.dx = 0
    Tail.dy = -1
  elseif game_map[ Tail.x ][ Tail.y ] == "down" then
    Tail.dx = 0
    Tail.dy = 1
  end

  if (((Head.x < 0) or (Head.x > xMax)) or ((Head.y < 0) or (Head.y > yMax))) then
    return 
  elseif game_map[ Head.x ][ Head.y ] == "food" then
    eat_food()
  else
    lcd.drawPixmap(Tail.x * 6, Tail.y * 8,"/SCRIPTS/TELEMETRY/nohead.bmp" )    -- new CMc 20170203
    game_map[ Tail.x ][ Tail.y ] = nil
    Tail.x = Tail.x + Tail.dx
    Tail.y = Tail.y + Tail.dy
  end

  game_map[ Head.x ][ Head.y ] = direction
  Head.x = Head.x + Head.dx
  Head.y = Head.y + Head.dy

  lcd.drawPixmap(Head.x * 6, Head.y * 8,"/SCRIPTS/TELEMETRY/snakehd.bmp" )    -- new CMc 20170203
end 

-- ************************************************************************************************

local function init_function()
  food = false
  firstTime = true
  size = 3
  score = 0
  Tail.x = 1
  Tail.y = 1
  Head.x = Tail.x + ( size - 1 )
  Head.y = Tail.y
  Head.dx = 1
  Head.dy = 0
  Tail.dx = Head.dx
  Tail.dy = Head.dy
  direction = "right"
  exitLoop = false
  
end

-- ************************************************************************************************


local function run_function(event)
    
  if event == nil then
    raise("Cannot be run as a model script!")
  end
  
-- ***************************************************************************
--   Press Exit button to return to main Taranis screen
-- ***************************************************************************
  if event == EVT_EXIT_BREAK then
    firstTime = true
    return 2
  end

-- ****************************************************************************
-- Change of Snake speed CMc  20161007
--   Press Plus button to increase Snake speed decreasing value of snakeSpeed 
--   Press Minus button to decrease Snake speed increasing value of snakeSpeed
-- ****************************************************************************
 
  if event == EVT_PLUS_BREAK then  
      snakeSpeed = snakeSpeed - speedStep
      if snakeSpeed < speedMax then
        snakeSpeed = snakeSpeed + speedStep
      end
  end

  if event == EVT_MINUS_BREAK then  
      snakeSpeed = snakeSpeed + speedStep
      if snakeSpeed > speedMin then
        snakeSpeed = snakeSpeed - speedStep
      end
  end

-- **************************************************************************
 
    if firstTime == true then  -- This section is executed once when the telemetry screen is first
                             -- activated
      firstTime = false
      exitLoop = false
      score = 0
      delayCounter = snakeSpeed    -- namechange to delayCounter  CMc  20161005
      lcd.clear()

      for i = 0, xMax, 1 do
        game_map[ i ] = {}
      end
    
      for i = 0, size - 1, 1 do
        game_map[ Tail.x + ( i * Tail.dx ) ][ Tail.y + ( i * Tail.dy ) ] = direction
        lcd.drawPixmap((Tail.x + ( i * Tail.dx ) ) * 6, ( Tail.y + ( i * Tail.dy ) ) * 8,"/SCRIPTS/TELEMETRY/snakehd.bmp" )    -- new CMc 20170203
     end
    
      create_food()
  end

-- **************************************************************************
  if exitLoop == false then
   if delayCounter > 0 then     -- delay loop controls speed of the snake
       delayCounter = delayCounter - 1   
       return
    end
    delayCounter = snakeSpeed

    getDirection()
    move()
  end

  if check_collision() then
    exitLoop = true
    lcd.clear()
    -- Display Score and Reset instruction
    lcd.drawRectangle(LCD_W/2-90, LCD_H/2-24, 180, 48,SOLID)
    lcd.drawText(LCD_W/2-22, LCD_H/2-20, "Score: ", 0)
    lcd.drawNumber(LCD_W/2+24, LCD_H/2-20, score, 0)
    lcd.drawText(LCD_W/2-84, LCD_H/2-6, "Aileron Left or Right to Restart", 0)
    lcd.drawText(LCD_W/2-60, LCD_H/2+10,"+ or - to Change Speed", 0)

    -- command to start another snake.  Aileron moved more than 20% Left or Right
    if math.abs(getValue("ail")) < 200  then
       return
    end

   -- set up to do another snake
      firstTime = true
      exitLoop = false
      Tail.x = 1
      Tail.y = 1
      Head.x = Tail.x + ( size - 1 )
      Head.y = Tail.y
      Head.dx = 1
      Head.dy = 0
      Tail.dx = Head.dx
      Tail.dy = Head.dy
      direction = "right"
   end

end

return {run=run_function, init=init_function}
